package pogr;

import java.awt.Canvas;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JLabel;

/**
 * Abstraktní třída, která je předkem všech vyplňovacích algoritmů. Obshauje některé společné funkce pro všechny algoritmy
 *
 * @author Kamil Ondrák
 */
public abstract class Algoritmus {
    /**
     * Seznam všech vrcholů v zadaném polygonu
     */
    protected ArrayList<Point> polygon;
    /**
     * Plátno, do kterého probíhá vykreslování vygebnereovaného obrazu
     */
    protected MujCanvas platno;
    /**
     * Label pro zprávy pro uživatele
     */
    protected JLabel labZpravy;
    /**
     * Graphics pro kreslení vygenerovaného obrázku
     */
    protected Graphics g;
    /**
     * barva, kterou probíhá kreslení
     */
    protected int barva;
    /**
     * Obrázek, do kterého se kreslí
     */
    protected BufferedImage obrazek;

    private final int VELBODU=3;

    /**
     * Zobrazuje zprávu uživateli
     *
     * @param s zpráva
     */
    protected final void zprava(String s) {
        labZpravy.setText(s);
    }

    /**
     * Nastavuje barvu, kterou bude dále prováděno kreslení
     *
     * @param _barva barva
     *
     */
    protected void setBarva(int _barva) {
        barva=_barva;
    }

    /**
     * Vykreslujekřížek v zadaném bodě a zobrazi aktualizaný obraz
     * @param b Bod
     */
    protected void vykresliBod(Point b) {
        setBarva(0x0000FF);
        vykresliUsecku(b.x-VELBODU, b.y-VELBODU, b.x + VELBODU, b.y + VELBODU);
        vykresliUsecku(b.x-VELBODU, b.y+VELBODU, b.x + VELBODU, b.y - VELBODU);
        platno.setObrazek(obrazek);
        platno.repaint();
    }

    /**
     * Vykreslí úsečku DDA algoritmem. Používá aktuílní hodnotu proměnné barva
     * @param x1    x-ová souřadnice prvního vrcholu
     * @param y1    y-ová souřadnice prvního vrcholu
     * @param x2    x-ová souřadnice druhého vrcholu
     * @param y2    y-ová souřadnice druhého vrcholu
     */
    protected void vykresliUsecku(int x1, int y1, int x2, int y2) {
        double m = (double) (y2 - y1) / (double) (x2 - x1);
        if (Math.abs(m) <= 1) {
            if (x1 > x2) {
                int x3 = x1;
                int y3 = y1;
                x1 = x2;
                y1 = y2;
                x2 = x3;
                y2 = y3;
            }
            int x = x1;
            double y = y1;
            while (x <= x2) {
                obrazek.setRGB(x, (int) Math.round(y), barva);
                x++;
                y += m;
            }
        }
        else {
            if (y1 > y2) {
                int x3 = x1;
                int y3 = y1;
                x1 = x2;
                y1 = y2;
                x2 = x3;
                y2 = y3;
            }

            double x = x1;
            int y = y1;
            while (y <= y2) {
                obrazek.setRGB((int) Math.round(x), y, barva);
                x += 1/m;
                y++;
            }
        }
    }

    /**
     * Konstruktor třídy Algoritmus
     *
     * @param _platno       Plátno (Canvas) do kterého se bude kreslit
     * @param _labZpravy    Label, do kterého budou zobrazovány zprávy pro uživatele
     */
    public Algoritmus(MujCanvas _platno, JLabel _labZpravy) {
        platno = _platno;
        labZpravy=_labZpravy;
        platno.setObrazek(obrazek);
        platno.repaint();

        polygon = new ArrayList<Point>();
        g = platno.getGraphics();

        obrazek = new BufferedImage(platno.getWidth(), platno.getHeight(), BufferedImage.TYPE_INT_RGB);
        for (int i = 0; i < obrazek.getWidth(); i++) {
            for (int j = 0; j < obrazek.getHeight(); j++) {
                obrazek.setRGB(i, j, 0xFFFFFF);
            }
        }

        zprava("Myší určte vrcholy polygonu");
    }

    /**
     * Přidá bod k polygonu a vykreslí ho
     * @param b Bod
     */
    public void pridejBod(Point b) {
        polygon.add(b);
        vykresliBod(b);
    }

    /**
     * Vykreslí polygon dle dosud zadaných vrcholů. Pokud jsou zadány méně než tři vrcholy, vypíše zoznámení a polygon nevykresluje
     * @return  true pokud byl polygon vykreslen, false pokud ne
     */
    public boolean vykresliPolygon() {
        if (polygon.size()<3) {
            zprava("Polygon musí mít alespoň tři vrcholy");
            return false;
        }
        polygon.add(polygon.get(0));
        setBarva(0xFF0000);
        for (int i=0;i<polygon.size()-1;i++) {
            System.out.print("Vrchol "+i+"\n");
            vykresliUsecku(polygon.get(i).x, polygon.get(i).y, polygon.get(i+1).x, polygon.get(i+1).y);
        }

        //g.drawImage(obrazek, 0, 0, io);
        platno.setObrazek(obrazek);
        platno.repaint();
        zprava("Pro vyplnění polygonu klikněte na příslušné tlačítko.");
        return true;


    }

    /**
     * Smaže aktuální polygon a plátno
     */
    public void smaz() {
        System.out.print("Mazeme zevnitr\n");
        polygon.clear();
        obrazek = new BufferedImage(platno.getWidth(), platno.getHeight(), BufferedImage.TYPE_INT_RGB);
        for (int i = 0; i < obrazek.getWidth(); i++) {
            for (int j = 0; j < obrazek.getHeight(); j++) {
                obrazek.setRGB(i, j, 0xFFFFFF);
            }
        }
        platno.setObrazek(obrazek);
        platno.repaint();
    }

    public void obnov() {
        obrazek.setRGB(100, 100, 0x0000FF);
        try {
            //g.drawImage(obrazek, 0, 0, io);
            Thread.sleep(500);
        } catch (InterruptedException ex) {
            Logger.getLogger(Algoritmus.class.getName()).log(Level.SEVERE, null, ex);
        }
        //g.drawImage(obrazek, 0, 0, io);
        System.out.print("Obnoveno!\n");
    }

    /**
     * Abstraktní třída pro samotné vyplnění polygonu
     * @param postup Pokud je true, bude průběžně vykreslován postup vyplňování na obrazovku
     */
    public abstract void vyplnPolygon(boolean postup);

}
